#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "cas.h"

/* $Id: non_ssl_client.c,v 1.9 2006-04-08 15:14:23+09 naito Exp $ */

static void log(const char *msg) ;

char userid[1024] ;

int CAS_validate_non_SSL(char *ticket, char *service, char *outbuf, int outbuflen, mod_cas2_conf *c)
{
  int err, b, ret, total;
  int s = 0;
  struct sockaddr_in sa;
  struct hostent h, *hp2;
  char buf[BUFLEN];
  char *full_request, *str, *tmp;
  size_t full_request_size ;

  LOG("in CAS_validate_non_SSL()\n") ;

  hp2 = gethostbyname(c->CASHost) ;
  if ((s = socket(AF_INET, SOCK_STREAM, 0)) == -1) return CAS_FAIL_OTHER ;
  sa.sin_port = htons(atoi(c->CASValidatePort)) ;
  sa.sin_family = AF_INET ;
  memcpy(&h, hp2, sizeof(h));
  memcpy(&(sa.sin_addr.s_addr), h.h_addr_list[0], hp2->h_length); 
  if (connect(s, (struct sockaddr*) &sa, sizeof(sa)) == -1) {
    close(s) ;
    return CAS_FAIL_OTHER ; // FAIL ;
  }

  /* full_request= "CASMethod /CASValidate?ticket=ticket&service=service\n\n" */
  full_request_size 
    = strlen(c->CASMethod) + strlen(" ") + strlen(c->CASValidate) + strlen("?ticket=") 
    + strlen(ticket) + strlen("&service=") + strlen(service) + strlen(" ") + strlen(c->CASProtocol) 
    + strlen("\n\n") + (size_t) 10 ; 
  full_request = (char *)malloc(full_request_size) ;
  sprintf(full_request, "%s %s?ticket=%s&service=%s %s\n\n",
	  c->CASMethod, c->CASValidate, ticket, service, c->CASProtocol);
  LOG("full_request: ") ; LOG(full_request) ; LOG("\n") ;
  if (!write(s, full_request, strlen(full_request))) {
    close(s) ;
    free(full_request) ;
    return CAS_FAIL_OTHER ;
  }
  free(full_request) ;
  total = 0 ;
  do {
    b = read(s, buf + total, (sizeof(buf) - 1) - total) ;
    total += b ;
    if (total > BUFLEN) return CAS_FAIL_OTHER ;
  } while(b > 0) ;
  buf[total] = '\0' ;
  close(s) ;
  if (b != 0 || total >= sizeof(buf) - 1) return CAS_FAIL_OTHER ;
  LOG("buf: ") ; LOG(buf) ; LOG("\n") ;
  return check_cas_reply(buf, outbuf, c->CASMatchValidUser) ;
  /*
  if (outbuflen < strlen(str) + 1) return 0 ;
  strcpy(outbuf, buf); 
  */
  /* , ticket Ȥ check  */
}


static void log(const char *msg) { 
  FILE *fp ;
  fp = fopen("/tmp/log", "a");
  fputs(msg, fp);
  fflush(fp);
  fclose(fp) ;
}

int check_cas_reply(char *buf, char *outbuf, char *CAS_UID)
{
  #define	CAS_AUTH_FAILUER	"<cas:authenticationFailure"
  #define	CAS_AUTH_SUCCESS	"<cas:authenticationSuccess"
  char	*p, *q, *r ;
  char	test[BUFLEN] ;
  char  inword[BUFLEN], outword[BUFLEN] ;

  bzero(test,BUFLEN) ;
  if ((p = strstr(buf, CAS_AUTH_FAILUER)) != NULL) {
    LOG("AUTH: ") ; LOG(p) ; LOG("\n") ;
    q = strchr(p, '\'') ;
    r = strchr(q+1, '\'') ;
    strncpy(test, q+1, r-q-1) ;
    LOG("AUTH: ") ; LOG(test) ; LOG("*\n") ;
    if (strcmp(test, "INVALID_SERVICE") == 0) return CAS_FAIL_INVALID_SERVICE ;
    if (strcmp(test, "INVALID_TICKET") == 0) return CAS_FAIL_INVALID_TICKET ;
    if (strcmp(test, "ACCESS_DENIED") == 0) return CAS_FAIL_ACCESS_DENIED ;
    return CAS_FAIL_OTHER ;
  }
  if ((p = strstr(buf, CAS_AUTH_SUCCESS)) == NULL) {
    return CAS_FAIL_OTHER ;
  }
  // SUCCESS ;
  bzero(test,BUFLEN) ;
  sprintf(inword,"<cas:%s>", CAS_UID) ;
  sprintf(outword,"</cas:%s>", CAS_UID) ;
  if ((p = strstr(buf, inword)) != NULL) {
    r = p + strlen(inword) ;
    q = strstr(r,outword) ;
    if (q != NULL) {
      char temp[10] ;
      sprintf(temp,"wordlen = %d\n", q-r) ;
      strncpy(outbuf, r, q-r) ;
      outbuf[q-r] = 0 ;
    }
    else {
      outbuf[0] = 0 ;
    }
  }
  return CAS_SUCCESS ;
}
